function trainSVM(dataInput, kernelInput, paramInput, modelInput)

% TRAINSVM
%      trainSVM is the main function of the toolbox. 
%
%      trainSVM(dataInput, kernelInput, paramInput, modelInput) stores the
%      results of the SVM training in a global structure called 'svModel'
%      that has to be declared before calling trainSVM.
%
%      INPUT 
%           dataInput is a structure obtained via the function 'data' and
%           contains the training and the testing set,
%
%           kernelInput is a structure obtained via the function 'kernel'
%           and contains the type and parameters of the kernel,
%
%           paramInput is a structure obtained via the function 'param' and
%           contains all the parameters for the training settings, such as
%           precision, slack variable, type of training (multiclass,
%           oneclass, ...),
%
%           modelInput is a structure obtained via the function 'model'.
%           This parameter is optional and used to initialize a specific a
%           priori solution. It contains the support vectors and their
%           coefficient.
%
%      Example
%           global svModel
%           [x,y,xt,yt]=dataset('Checkers',200,10,0.5); % generates data
%           donnees = data(x,y,xt,yt);                  % stores data
%           noyau = kernel('rbf',.9);                   % stores kernel
%           parametres = param(500,50,'binary','chol'); % stores parameters
%           trainSVM(donnees, noyau, parametres);       % train the SVM
%           prediction = testSVM;                       % gives the results
%                                                       % on the test set
%                                                       
%
% See also: testSVM, data, kernel, param, model, svm, dataset

% Last Modified by Gaelle Loosli 11-Feb-2005

%=============================
% checking input
if nargin>4
    error('too many input arguments');
end
if nargin<2
    error('missing argument');
end
if nargin<=2
    paramInput = param;
end
%=============================



%=============================
% variables
global svModel;
global temporarelyDataNeeded;
toolpath = which('trainSVM');
toolpath = toolpath(1:end-10);
labelMax = max(dataInput.trainlab);
labelMin = min(dataInput.trainlab);
if labelMin==labelMax
    nbclass = 1;
else
    nbclass = labelMax-labelMin+1;
end
if nargin<=3
    switch paramInput.multiclass
        case '1vs1'
            modelInput = model(nbclass*(nbclass-1)/2);
        case '1vsall'
            modelInput = model(nbclass);
        case '1class'
            modelInput = model(1);
        otherwise
            modelInput = model;
    end
end
svm(kernelInput,dataInput,paramInput,modelInput);
clear dataInput;
% model
pathMeth = [toolpath,svModel.param.method,'/'];
addpath(pathMeth);
if strcmp(svModel.param.type,'nu')
    pathMeth = [toolpath,svModel.param.method,'/nu/'];
    addpath(pathMeth);
end
if strcmp(svModel.param.type,'DC')
    pathMeth = [toolpath,svModel.param.method,'/DC/'];
    addpath(pathMeth);
end
if strcmp(svModel.param.type,'VO')
    pathMeth = [toolpath,svModel.param.method,'/vo/'];
    addpath(pathMeth);
end

%=============================
svModel.model.iclass = 0;
switch svModel.param.multiclass
    case '1vs1'
        svModel.data.trainlab_t = svModel.data.trainlab;
        svModel.data.trainvec_t = svModel.data.trainvec;
        for i = labelMin:labelMax
            for j = i+1:labelMax
                svModel.model.iclass = svModel.model.iclass+1;
                global temporarelyDataNeeded;
                if svModel.param.verbose>0
                    fprintf('classe %d contre %d : \n',i,j);
                end
                svModel.data.trainvec = svModel.data.trainvec_t(:,[find(svModel.data.trainlab_t==i);...
                    find(svModel.data.trainlab_t==j)]);
                if strcmp(svModel.param.type ,'invariant') | strcmp(svModel.param.type ,'uncertain')
                    svModel.data.trainvec = [svModel.data.trainvec,zeros(size(svModel.data.trainvec,1),500)];
                end
                svModel.data.trainlab = [ones(length(find(svModel.data.trainlab_t==i)),1);...
                    -ones(length(find(svModel.data.trainlab_t==j)),1)];
                length_temp = length(svModel.data.trainlab);
                main;
                svModel.data.trainvecInv{svModel.model.iclass} = ...
                    svModel.data.trainvec(:,length_temp+1:temporarelyDataNeeded.n);
                svModel.data.trainlabInv{svModel.model.iclass} = ...
                    svModel.data.trainlab(length_temp+1:temporarelyDataNeeded.n);
                svModel.data.trainlabInv{svModel.model.iclass}...
                    (find(svModel.data.trainlabInv{svModel.model.iclass}==1)) = i;
                svModel.data.trainlabInv{svModel.model.iclass}...
                    (find(svModel.data.trainlabInv{svModel.model.iclass}==-1)) = j;
                svModel.param.C = paramInput.C;
                clear global temporarelyDataNeeded;
            end
        end
        svModel.data.trainlab = svModel.data.trainlab_t;
        svModel.data.trainvec = svModel.data.trainvec_t;
    case '1vsall'
        svModel.data.trainlab_t = svModel.data.trainlab;
        if strcmp(svModel.param.type,'invariant') | strcmp(svModel.param.type ,'uncertain')
            svModel.data.trainvec_t = svModel.data.trainvec;
        end
        for i = labelMin:labelMax
            global temporarelyDataNeeded;
            svModel.model.iclass = svModel.model.iclass+1;
            if svModel.param.verbose>0
                fprintf('classe %d contre les autres : \n',i);
            end
            if strcmp(svModel.param.type ,'invariant') | strcmp(svModel.param.type ,'uncertain')
                svModel.data.trainvec = [svModel.data.trainvec_t,zeros(size(svModel.data.trainvec,1),500)];
            end
            svModel.data.trainlab = svModel.data.trainlab_t;
            svModel.data.trainlab(svModel.data.trainlab==i) = -1;
            svModel.data.trainlab(svModel.data.trainlab>=0) = 1;
            main;
            if strcmp(svModel.param.type ,'invariant') | strcmp(svModel.param.type ,'uncertain')
                svModel.data.trainvecInv{svModel.model.iclass} = ...
                    svModel.data.trainvec(:,length(svModel.data.trainlab_t)+1:temporarelyDataNeeded.n);
                svModel.data.trainlabInv{svModel.model.iclass} = ...
                    svModel.data.trainlab(length(svModel.data.trainlab_t)+1:temporarelyDataNeeded.n);
                svModel.data.trainlabInv{svModel.model.iclass}...
                    (find(svModel.data.trainlabInv{svModel.model.iclass}==1)) = -2;
                svModel.data.trainlabInv{svModel.model.iclass}...
                    (find(svModel.data.trainlabInv{svModel.model.iclass}==-1)) = i;
                svModel.data.trainlabInv{svModel.model.iclass}...
                    (find(svModel.data.trainlabInv{svModel.model.iclass}==-2)) = -1;
            end
            svModel.param.C = paramInput.C;
            clear global temporarelyDataNeeded;
        end
        svModel.data.trainlab = svModel.data.trainlab_t;
        if strcmp(svModel.param.type,'invariant') | strcmp(svModel.param.type ,'uncertain')
            svModel.data.trainvec = svModel.data.trainvec_t;
        end
    case '1class'
        svModel.data.trainlab = ones(size(svModel.data.trainvec,2),1);
        pathMeth = [toolpath,svModel.param.method,'/1class/'];
        addpath(pathMeth);
        svModel.model.iclass = svModel.model.iclass+1;
        main
    otherwise
        svModel.model.iclass = svModel.model.iclass+1;
        main
end
clear global temporarelyDataNeeded;


function main
global svModel;
global temporarelyDataNeeded;
%=============================
% main program
%createcache;
initialize;
temporarelyDataNeeded.countVS = zeros(1,length(svModel.data.trainlab));
while not(isWellClassified)
    %fprintf('point trait� : %d \n', temporarelyDataNeeded.new_active);
    solveLinearSystem
    if isWellConstrained(svModel.model.alpha)==0
        if strcmp(svModel.param.type ,'nu')
            projectSolutionNu
        elseif strcmp(svModel.param.type ,'lasvm')
            projectSolutionLasvm
        else
            projectSolution			% activates a constraint
        end
    else
        if strcmp(svModel.param.type ,'invariant')
            selectConstraintToRelaxInv
        elseif strcmp(svModel.param.type ,'uncertain')
            selectConstraintToRelaxUncertain
        elseif strcmp(svModel.param.type ,'lasvm')
            selectConstraintToRelaxLasvm
        else
            selectConstraintToRelax	% adds a point to the solution
        end
    end
    maj
end
if svModel.param.verbose>0
    fprintf(svModel.param.fid,'%d support vectors, %d C bounded \n',temporarelyDataNeeded.n_active,temporarelyDataNeeded.n_bounded);
end
svModel.model.nbC = svModel.model.nbC + temporarelyDataNeeded.n_bounded;
%=============================
